from ..functools import case


def get_model_names(model):
    return [model._meta.verbose_name, model._meta.verbose_name_plural]

def verbose_choices(*args):
    return map(lambda a: (a.string, a), args)

def verbose_choice(obj, f):
    return case(
        getattr(obj, f),
        dict(obj._meta.get_field(f).choices))

def verbose_attr(obj, field):
    return (obj._meta.get_field(field).verbose_name, getattr(obj, field))



from django.conf import settings
from django.utils.text import capfirst
from era import _
from era.components import register, TemplateComponent, Tag, Component, Break, Link, Image, Menu
from era.utils.functools import factual
from era.utils.translation.verbose import verbose_attr


class UserModelMixin:
    @property
    def is_superuser(self):
        return self.request.user.is_authenticated() \
            and self.request.user.role == settings.USER_ROLES[0].string

    def get_user_menu_item(self):
        return {
            'model': 'users.User',
            'caption': {'icon': 'users'},
            'include': self.is_superuser}


class AuthMenu(Menu):
    def get_items(self):
        if self.request.user.is_anonymous():
            return [{
                'caption': {'icon': 'sign-in', 'title': _('login')},
                'link': {'url': 'login'},
                'active': self.check_active('login', 'recovery', 'confirm')}]
        return [{
            'caption': {'icon': 'user', 'title': _('profile')},
            'link': {'url': 'profile'}
            }, {
            'caption': {'icon': 'sign-out', 'title': _('logout')},
            'link': {'url': 'logout'}}]


class BaseUsersMenu(UserModelMixin, AuthMenu):
    def get_items(self):
        return [self.get_user_menu_item()] + super().get_items()


@register.era
class Logo(Link):
    inline = True

    def resolve_props(self):
        return {
            'host': True,
            'url': 'index',
            'newtab': True}

    def get_nodelist(self):
        return self.inject(Image, {
            'host': True,
            'url': 'images/logo.png'})


class Notification(Component):
    parts = ['logo', 'empty', 'value', 'link']

    def get_parts(self):
        return self.parts

    def render_empty(self):
        return ' '

    def render_value(self, label=None, value=None):
        label = label or self.value
        return ': '.join([
            capfirst(str(label)),
            value or self.props.get(label.string)])

    def render_logo(self):
        if self.props.user.comm.display_html:
            return self.inject(Logo)

    def render_link(self, **kw):
        if self.props.user.comm.display_html:
            return self.inject(Link, dict({'host': True, 'newtab': True}, **kw))

    def DOM(self):
        delimiter = Break().as_string() if self.props.user.comm.display_html else '\n'
        return delimiter.join(factual(map(
            lambda attr: getattr(self, '_'.join(['render', attr]))(),
            self.get_parts())))


class ConfirmationMixin:
    value = _('code')

    def render_link(self):
        return super().render_link(
            url=self.url_name,
            qs={'code': self.props.code},
            nodelist=_('confirm'))


class JoinNotification(ConfirmationMixin, Notification):
    url_name = 'confirm'


class ResetNotification(ConfirmationMixin, Notification):
    url_name = 'unlock'


class InviteNotification(Notification):
    value = _('password')

    def render_link(self):
        return super().render_link(
            url='login',
            nodelist=_('login'))



import re
import sys
from django.utils.text import capfirst
from django.utils.translation import get_language, ugettext_lazy
from ..functools import factual


def _(string, context=None):
    t = ugettext_lazy(': '.join(factual([context, string])))
    t.string = string
    return t


def get_string(obj):
    return obj if isinstance(obj, str) else obj.string


def normalize(string):
    return ''.join(map(
        lambda e: e[0] and re.sub(r'([A-Z])', r'-\g<1>', e[1]) or e[1],
        enumerate(string))).lower()


def capitalize(string):
    return ''.join(map(capfirst, string.split('-')))


def inflect(string):
    inflector = sys.modules.get('.'.join([__package__ or __name__, get_language()]))
    return inflector and inflector.inflect(string) or string



from itertools import chain
import re

from django.conf import settings
from django.contrib import messages
from django.core.urlresolvers import resolve, reverse
from django.shortcuts import redirect
from django.utils.decorators import method_decorator
from django.views.generic.base import TemplateResponseMixin, View
from ..utils.translation import normalize
from ..utils.urls import dispatch_decorator


class BaseViewMixin:
    keywords = []

    @property
    def about(self):
        return ' '.join(chain(
            self.keywords,
            [re.sub(r'-view$', '', normalize(self.__class__.__name__))]))

    @property
    def url_match(self):
        return resolve(self.request.path)

    def send_message(self, level, content):
        content and getattr(messages, level)(self.request, content)

    def navigate(self, name):
        return redirect(reverse(name))

    def reload(self):
        return self.navigate(self.url_match.url_name)


class BaseView(BaseViewMixin, TemplateResponseMixin, View):
    decorators = []
    components = {}
    page_title = settings.TITLE

    @dispatch_decorator
    def dispatch(self, request, *args, **kwargs):
        return super().dispatch(request, *args, **kwargs)

    def get_decorators(self):
        return self.decorators

    def get(self, request, *args, **kwargs):
        return getattr(super(),  'get', lambda r: None)(request) \
            or self.render_to_response(self.get_context_data())

    def get_page_title(self, **kw):
        return self.page_title

    def get_media(self, **kw):
        return ''

    def get_components(self, **kw):
        return {}

    def get_context_data(self, **kw):
        return dict(
            getattr(super(), 'get_context_data', lambda **x: x)(**kw),
            codename=settings.CODENAME,
            page_title=self.get_page_title(**kw),
            media=self.get_media(**kw),
            components=dict(self.components, **self.get_components(**kw)))

    def get_template_names(self):
        return list(map(lambda x: x + '.html', [self.about, 'index']))


class DisplayAttrMixin:
    def display_attr(self, obj, attr, **kw):
        method = 'get_{0}_display'.format(attr)
        if isinstance(obj, self.model) and hasattr(self, method):
            return getattr(self, method)(obj)
        elif kw.get('model', True) and hasattr(obj, method):
            return getattr(obj, method)()
        elif kw.get('direct', True):
            return getattr(obj, attr)
        raise AttributeError('Missing display data for "{0}"'.format(attr))



from importlib import import_module
from itertools import chain
import json
import re

from classytags.arguments import MultiKeywordArgument
from classytags.core import Options
from classytags.core import Tag as ClassyTag
from django.template import Library, TemplateSyntaxError
from django.template.loader import render_to_string

from ..utils.functools import call, unpack_args, filter_dict, reduce_dict, map_values, \
    truthful, factual, pick, omit, factual
from ..utils.translation import normalize
from ..utils.urls import resolve, get_site_url

register = Library()
register.era = lambda cls: register.tag(normalize(cls.__name__), cls)


@register.era
class Import(ClassyTag):
    def __init__(self, parser, tokens):
        for module in tokens.contents.split(' ')[1:]:
            path = '.'.join(
                [module, 'components'] if not module.startswith('.') else \
                [module.split('.')[1], 'components'] + module.split('.')[2:])
            try:
                parser.tags.update(import_module(path).register.tags)
            except ImportError:
                raise TemplateSyntaxError('can not import {0}'.format(path))

    def render(self, *args, **kw):
        return ''


class HTMLTag:
    def __init__(self, el):
        self.el = str(el)

    def __str__(self):
        return self.el

    def modify_tag(self, diff):
        tag, *rest = self.el.split('>')
        self.el = '>'.join([diff(tag)] + rest)

    def modify_content(self, content, fmt):
        self.el = re.sub(r'>(.*)<', fmt.format(content), self.el)

    def after(self, content):
        self.el += content

    def before(self, content):
        self.el = content + self.el

    def append(self, content):
        self.modify_content(content, r'>\g<1>{0}<')

    def prepend(self, content):
        self.modify_content(content, r'>{0}\g<1><')

    def add_attr(self, name, val=None):
        self.modify_tag(lambda tag: ' '.join([
            tag.endswith('/') and tag.rstrip('/') or tag,
            val is None and name or '{0}="{1}"'.format(name, val),
            tag.endswith('/') and '/' or '']))

    def add_class(self, *cls):
        if not any(cls):
            return
        if not self.has_attr('class'):
            self.add_attr('class', ' '.join(cls))
        else:
            self.modify_tag(
                lambda tag: re.sub(
                    r'class="([^"]*)"',
                    r'class="\g<1> {0}"'.format(' '.join(cls)),
                    tag))

    def has_attr(self, name):
        return name in self.el.split('>')[0]

    def empty(self):
        self.el = ''


class Props(dict):
    def __getattr__(self, name):
        if name in self:
            return self.get(name)
        raise AttributeError('Missing prop: ' + name)


class RequestUrlMixin:
    def check_active(self, *args):
        return resolve(self.request.path).url_name in args

    def get_host(self, *args, **kw):
        return get_site_url(self.request, *args, **kw)


class Component(RequestUrlMixin, ClassyTag):
    def __init__(self, parser=None, tokens=None, context=None):
        self.set_context(context or {})
        self.blocks = {}
        self.set_options()
        parser and super().__init__(parser, tokens)

    @classmethod
    def as_string(cls, request=None, **kw):
        # TODO: move to view.show
        obj = cls(context={'request': request})
        obj.set_props(kw)
        return obj.render_dom()

    def inject(self, *args, **kw):
        # TODO: deprecate
        return self.show(*args, **kw)

    def insert(self, cls, props=None, nodelist=None, **kw):
        obj = cls(context=self.context)
        if nodelist is not None:
            kw['nodelist'] = nodelist
        obj.set_props(props, **kw)
        return obj

    def show(self, cls, props=None, nodelist=None, **kw):
        return self.insert(cls, props, nodelist, **kw).render_dom()

    def render_tag(self, context=None, mka=None, **kw):
        if not self.context and context:
            self.set_context(context)
        self.set_props(
            dict(mka, **kw),
            **map_values(
                lambda val: val.render(self.context),
                self.blocks))
        return self.render_dom()

    def build(self, args, prefix='render'):
        return map(lambda arg: call(getattr(self, '_'.join([prefix, arg]))), args)

    def get_defaults(self):
        return {}

    def get_class_set(self, *args, prefix='', include=''):
        return ' '.join(factual(
            chain([include], map(
                lambda k: '-'.join(factual([prefix, k])),
                truthful(pick(self.props, *args)).keys()))))

    def resolve_props(self):
        return {}

    def set_context(self, context):
        self.context = context
        self.request = self.context.get('request')

    def set_options(self, **kw):
        self.options = Options(MultiKeywordArgument('mka', required=False), **kw)

    def set_props(self, d, **kw):
        self.props = Props(
            self.get_defaults(),
            **dict(d or {}, **kw))
        self.props.update(self.resolve_props())

    def render_dom(self):
        self.tune()
        self.dom = HTMLTag(self.DOM())
        self.tweak()
        return str(self.dom)

    def DOM(self):
        raise NotImplementedError

    def tune(self):
        pass

    def tweak(self):
        if not 'attrs' in self.props and 'class' in self.props:
            self.dom.add_class(self.props['class'])


class ComplexComponent(Component):
    parts = []

    def get_defaults(self):
        return {'nodelist': ''}

    def set_options(self, **kw):
        return super().set_options(**(kw or {'blocks': chain(
            [] if not self.parts else [(self.parts[0], 'nodelist')],
            [] if not self.parts else list(map(unpack_args(
                lambda i, b: (b, self.parts[i - 1])),
                enumerate(self.parts[1:], start=1))),
            [(
                'end-' + normalize(self.__class__.__name__),
                not self.parts and 'nodelist' or self.parts[-1])])}))


class TemplateComponent(Component):
    template = 'index.html'

    def render_dom(self):
        return render_to_string(self.template, dict(self.context, **self.props))


@register.era
class Tag(ComplexComponent):
    el = 'div'
    nobody = False
    inline = False
    named = True

    @property
    def root(self):
        return Tag

    def get_nodelist(self):
        return self.props.nodelist

    def resolve_attrs(self):
        return {}

    def resolve_node_name(self):
        return ' '.join(map(
            lambda cls: normalize(cls.__name__),
            filter(
                lambda cls: getattr(cls, 'named', False),
                self.__class__.__mro__[:self.__class__.__mro__.index(self.root)])))

    def resolve_tag(self):
        attrs = dict(
            self.props.get('attrs', {}),
            **self.resolve_attrs())
        attrs.update(filter_dict(
            lambda k, v: v, {
            'id': self.props.get('id'),
            'class': ' '.join(factual([
                self.named and self.resolve_node_name(),
                attrs.get('class', ''),
                self.props.get('class', '')]))}))
        return {
            'el': self.props.get('el', getattr(self, 'el')),
            'attrs': ' '.join(reduce_dict(
                lambda k, v: v and '{0}="{1}"'.format(k, v) or k,
                attrs))}

    def set_options(self, **kw):
        return super().set_options(**dict(kw, **(
            (self.inline or self.nobody) and {'blocks': []} or {})))

    def DOM(self):
        return ''.join([
            '<{el} {attrs}',
            '/>' if self.nobody else '>{nodelist}</{el}>']) \
        .format(**self.props)

    def tune(self):
        self.props.update(self.resolve_tag())
        not self.nobody and self.props.update({'nodelist': self.get_nodelist()})

    def tweak(self):
        pass


class ScriptedTag(Tag):
    @property
    def root(self):
        return ScriptedTag

    def get_script(self):
        return self.__class__.__name__

    def resolve_script(self):
        return {}

    def DOM(self):
        return ''.join(factual([
            super().DOM(),
            self.get_script() is not None and self.inject(
                Tag, {'el': 'script'}, '$(function() {{{0}}})'.format(
                    ''.join([
                        self.get_script(),
                        '({0})'.format(', '.join([
                            '\'.{0}\''.format(self.resolve_node_name().split(' ')[-1]),
                            json.dumps(self.resolve_script())]))])))]))


@register.era
class Display(Component):
    def resolve_props(self):
        return {'cls': self.context['components'].get(self.props.component, None)}

    def DOM(self):
        if self.props.cls:
            return self.show(self.props.cls, omit(self.props, 'component', 'cls'))
        return ''


@register.era
class Content(Tag):
    el = 'main'
    inline = True

    def get_nodelist(self):
        return self.show(Display, {'component': 'content'})



from django.conf import settings
from django.conf.urls import patterns, include, url
from django.conf.urls.static import static
from django.contrib import admin
from django.utils.module_loading import import_string

from .utils.translation import _
from .utils.urls import exists_import, http_error


try:
    from app.urls import urlpatterns as index
    urlpatterns = index
except ImportError:
    urlpatterns = patterns('')

if not 'index' in [x.name for x in urlpatterns]:
    urlpatterns += patterns('', url(
        r'^$',
        import_string(settings.INDEX_VIEW).as_view(),
        name='index'))

urlpatterns += patterns('', *list(map(
    lambda app: url(r'', include(app + '.urls')),
    filter(
        lambda app: not app in ['app', __package__] \
            and exists_import('.'.join([app, 'urls'])),
        settings.MODULES))))

if settings.DEBUG:
    urlpatterns += static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)


handler403 = http_error(403, _('None available'))
handler404 = http_error(404, _('Page not found'))
handler500 = http_error(500, _('Server error'))



import os
import sys
import django

DEBUG = True
TEMPLATE_DEBUG = DEBUG
ALLOWED_HOSTS = []

SECRET_KEY = 'era'
TEST_RUNNER = 'era.tests.runner.NoDbDiscoverRunner'
EMAIL_BACKEND = 'django.core.mail.backends.console.EmailBackend'

BASE_DIR = os.path.join(os.path.dirname(os.path.dirname(sys.modules.get('app').__file__)), '..')
CODENAME = os.path.basename(os.path.abspath(BASE_DIR))
INDEX_VIEW = 'app.views.IndexView'
MODULES = ['app']

USE_I18N = True
LOCALE_PATHS = (
    os.path.join(BASE_DIR, 'locales'),
    os.path.join(os.path.dirname(django.__file__), 'contrib', 'admin', 'locale')
)

DATE_FORMAT = 'd.m.y'
TIME_FORMAT = 'H:M'
DATETIME_FORMAT = 'd.m.y H:M'
DATE_INPUT_FORMATS = ['%d.%m.%y']
DATETIME_INPUT_FORMATS = ['%d.%m.%y %H:%M']

STATIC_ROOT = os.path.join(BASE_DIR, 'static')
STATIC_URL = '/static/'
MEDIA_ROOT = os.path.join(BASE_DIR, 'uploads')
MEDIA_URL = '/uploads/'

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.postgresql_psycopg2',
        'NAME': CODENAME,
    }
}

INSTALLED_APPS = [
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'djangobower',
    'era'
]

BOWER_COMPONENTS_ROOT = BASE_DIR
BOWER_INSTALLED_APPS = [
    'moment',
    'jquery#2',
    'underscore',
    'bootstrap',
    'seiyria-bootstrap-slider',
    'eonasdan-bootstrap-datetimepicker',
    'font-awesome',
    'jquery-query-object'
]

MIDDLEWARE_CLASSES = [
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.auth.middleware.SessionAuthenticationMiddleware',
    'django.middleware.locale.LocaleMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

TEMPLATE_LOADERS = [
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader'
]

TEMPLATE_CONTEXT_PROCESSORS = [
    'django.contrib.auth.context_processors.auth',
    'django.core.context_processors.request',
    'django.contrib.messages.context_processors.messages'
]

STATICFILES_FINDERS = [
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
    'djangobower.finders.BowerFinder'
]

MESSAGE_STORAGE = 'django.contrib.messages.storage.session.SessionStorage'
ROOT_URLCONF = 'era.urls'
WSGI_APPLICATION = 'app.wsgi.application'



from .functools import unidec


@unidec
def o_O(fn, *args, **kw):
    import ipdb
    locals().update(fn.__globals__)
    closure = [x.cell_contents for x in fn.__closure__ or []]
    ipdb.set_trace()
    return fn(*args, **kw)


__builtins__['o_O'] = o_O
